/****************************************************************************/
/*                                                                          */
/* FONTEDIT.C                                                               */
/*                                                                          */
/*                                                                          */
/****************************************************************************/
/*----------------------------- include files ------------------------------*/
#include <stdio.h>
#include <ctype.h>
#include "t.h"                                /* include the TUI header     */
#include "font.def"                           /* include dialog definitions */

/*--------------------------------- defines --------------------------------*/
#define MAX_UNDO_LEVEL 64

/*----------------------------- prototypes ---------------------------------*/
int main( void );
void draw_font_set( void );
void draw_edit_line( int x_off, int y_off, int line, int att, WINDOW *wnp, uchar val );
void do_editor( uchar *font, int scan_lines );
void undo_stack( uchar *p, int scan_lines );
void undo_pop( uchar *p, int scan_lines );
void fonthelp( int inx );
int  register_font_tui( TUI *tuip );
void register_font_dlg0( TUI *tuip );

/*--------------------------------- globals --------------------------------*/
TUI     Font_tui, *Tuip = &Font_tui;

uchar   Font_buff[256 * 16];
uchar   Undo[MAX_UNDO_LEVEL][16], Undo_inx;

uchar   Bits[9]  = { 0x01, 0x02, 0x04, 0x08, 0x10, 0x20, 0x40, 0x80, 0 };
uchar   Maskr[9] = { 0xff, 0x7f, 0x3f, 0x1f, 0x0f, 0x07, 0x03, 0x01, 0 };
uchar   Maskl[9] = { 0x00, 0x80, 0xc0, 0xe0, 0xf0, 0xf8, 0xfc, 0xfe, 0 };

char    Path[81], Filename[81], Fullname[81];

/*------------------- slider/image/palette globals -------------------------*/
uchar Palette_font[16] =
  { 0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x14, 0x07,
    0x38, 0x39, 0x3A, 0x3B, 0x3C, 0x3D, 0x3E, 0x3F };

/*********/
/* ~main */
/*       ********************************************************************/
/****************************************************************************/
int main( void )
{
  int scan_lines, rows, x;
  uchar far *f, *np;

  if ( register_font_tui(Tuip) )
  /* if ( load_tui(Tuip, "FONT.TUI") ) */
  {
    init_tui(Tuip);
    init_clock(0x3333);
    if( EgaVga )
    {
      m_hide();
      wn_plst( CENTERED, V_rows-1,
      "Click right button on desired command for Help or Alt-H for Help Index", Tuip->back_wnp);
      if ( Ega )
      {
        strcpy(Filename, "8x14.FNT");
        rom8x14(0,1);
        rom8x14(1,1);
        f = get_font_info( 2, &scan_lines, &rows );
      } else
      {
        strcpy(Filename, "8x16.FNT");
        rom8x16(0,1);
        rom8x16(1,1);
        f = get_font_info( 6, &scan_lines, &rows );
      }
      np = Font_buff;
      for( x = 0; x < scan_lines * 256; x++ )
        *np++ = *f++;
      install_font( Font_buff, 0, 0, 256, scan_lines, 1 );
      install_font( Font_buff, 1, 0, 256, scan_lines, 1 );
      set_block_ab(0,1);
      m_show();
      draw_dialog(Tuip, FONT_DLG);
      draw_font_set();
      do_editor(Font_buff, scan_lines);
      erase_dialog(Tuip, FONT_DLG);
    } else
    {
      wn_plst(0, 0, "Sorry, you must have an EGA/VGA board!", Tuip->back_wnp);
      wait_event();
    }
    end_clock();
    end_tui(Tuip);
    free_tui(Tuip);
  } else printf("Sorry, could not register the TUI\n");
  return(1);
}
/*** end of main ***/

/******************/
/* ~draw_font_set */
/*                ***********************************************************/
/****************************************************************************/
void draw_font_set( void )
{
  WINDOW *wnp = get_dlg_wnp(Tuip, FONT_DLG);      /* get the dlg window ptr */
  int x = get_item_x(Tuip, FONTBOX);              /* get the fontbox x      */
  int y = get_item_y(Tuip, FONTBOX);              /* get the fontbox y      */
  uchar att = get_box_att(Tuip, FONTBOX);         /* get the fontbox color  */
  uchar save_att = wnp->att;
  int i;

  x -= wnp->inside;
  y -= wnp->inside;
  wn_att(att, wnp);
  for( i = 0; i < 256; i++ )
  {
    mv_cs( x + (i % 16), y + (i / 16), wnp );
    wn_ch( (uchar) i, wnp );
  }
  wn_att(save_att, wnp);
}
/*** end of draw_font_set ***/

/*******************/
/* ~draw_edit_line */
/*                 **********************************************************/
/****************************************************************************/
void draw_edit_line( int x_off, int y_off, int line, int att, WINDOW *wnp, uchar val )
{
  int i, save_att = wnp->att;
  uchar ch;

  wnp->att = att;
  m_hide();
  for( i = 0; i < 8; i++ )
  {
    mv_cs( x_off + (i*2), y_off + line, wnp );
    ch = (uchar) ((val & 0x80) ? 178 : ' ');
    wn_ch(ch, wnp), wn_ch(ch, wnp);
    val <<= 1;
  }
  m_show();
  wnp->att = save_att;
}
/*** end of draw_edit_line ***/

/**************/
/* ~do_editor */
/*            ***************************************************************/
/****************************************************************************/
void do_editor( uchar *font, int scan_lines )
{
  WINDOW *wnp = get_dlg_wnp(Tuip, FONT_DLG);      /* get the dlg window ptr */
  int   font_x = get_item_x(Tuip, FONTBOX);       /* get the font box x     */
  int   font_y = get_item_y(Tuip, FONTBOX);       /* get the font box y     */
  int   edit_x = get_item_x(Tuip, EDITBOX);       /* get the edit box x     */
  int   edit_y = get_item_y(Tuip, EDITBOX);       /* get the edit box y     */
  int   att = get_box_att(Tuip, EDITBOX);         /* get the edit box color */

  M_LOC m;
  uchar save[32], tmp, tmp2, tmp3, left, right, *p, *p2, last_char = 'A', ch;
  uchar far *f, far *fp, *np;
  int   i, x, y, c = 'A', cx = 0, cy = 0, inx, save_att, temp_lines, rows,
        old_col, old_row, doset, disp_flag = ON, end_flag = OFF, event_rtn;
  ulong col_mask;

  font_x -= wnp->inside;
  font_y -= wnp->inside;
  edit_x -= wnp->inside;
  edit_y -= wnp->inside;
  setmem(save, 32, 0);
  while(!end_flag)
  {
    if (disp_flag)
    {
      csr_hide();
      m_hide();
      mv_cs(edit_x + 11, edit_y - 1, wnp);
      wn_ch( (uchar) c, wnp);
      save_att = wnp->att, wnp->att = att;
      for( y = 0; y < scan_lines; y++ )
      {
        tmp = *(font + (c * scan_lines) + y);
        for( x = 0; x < 8; x++ )
        {
          mv_cs( edit_x + (x*2), edit_y + y, wnp );
          ch = (uchar) ((uchar) (tmp & 0x80) ? 178 : ' ');
          wn_ch(ch, wnp);
          wn_ch(ch, wnp);
          tmp <<= 1;
        }
      }
      mv_cs( edit_x + (cx * 2), edit_y + cy, wnp );
      wn_ch(219, wnp), wn_ch(219, wnp);
      wnp->att = save_att;
      m_show();
      disp_flag = OFF;
    }
    m_hide();
    push(wnp->att);
    wnp->att = Tuip->base[FONTBOX].p_att;
    mv_cs( font_x + (last_char % 16), font_y + (last_char / 16), wnp );
    wn_ch( last_char, wnp );
    wnp->att = (LIGHTGRAY << 4) | LIGHTRED;
    mv_cs( font_x + (c % 16), font_y + (c / 16), wnp );
    wn_ch( (uchar) c, wnp );
    pop(wnp->att);
    if( c != (int) last_char )
      Undo_inx = 0;
    last_char = c;
    m_show();
    event_rtn = 0;
    Event.key = 0;
    inx = do_dialog(Tuip, FONT_DLG, DO_RETURN_EVENT | DO_RETURN_CMD);
    if( ((Event.m_button == RB)    && (inx != EDITBOX)) ||
        (( Event.key == KEY_ALT_H) && (inx == EVENT_RETURNED)) )
      fonthelp(inx);
    else switch(inx)
    {
      /*----------------------------- load font ----------------------------*/
      case FONT_LOAD:
        if ( file_selected("*.fnt", Path, Filename, Fullname) )
        {
          m_hide();
          temp_lines = load_font(font, Fullname);
          if (temp_lines)
          {
            scan_lines = temp_lines;
            switch(scan_lines)
            {
              case  8: rom8x8(0,1),  rom8x8(1,1);  break;
              case 14: rom8x14(0,1), rom8x14(1,1); break;
              case 16: if( Vga ) rom8x16(0,1), rom8x16(1,1); break;
            }
            install_font( font, 1, 0, 256, scan_lines, 1 );
            set_block_ab(0, 1);
            cx = cy = 0;
            draw_item(Tuip, EDITBOX);
          }
          m_show();
        }
        disp_flag = ON;
        break;

      /*--------------------------- font save as ---------------------------*/
      case FONT_SAVE_AS:
        if ( file_selected("*.fnt", Path, Filename, Fullname) )
          save_font(font, Fullname, scan_lines);
        break;

      /*----------------------------- font save ----------------------------*/
      case FONT_SAVE:
        save_font(font, Fullname, scan_lines);
        break;

      /*--------------------------- quit editor ----------------------------*/
      case FONT_EXIT:
        end_flag = ON;
        break;

      /*---------------- ESC pressed or clicked off dialog -----------------*/
      case NO_SELECTION:
        break;

      /*--------------------- clicked on a new character -------------------*/
      case FONTBOX:
        x = Event.m_x - font_x - wnp->pane.x_min - wnp->inside;
        y = Event.m_y - font_y - wnp->pane.y_min - wnp->inside;
        c = x + (y * 16);
        disp_flag = ON;
        break;

      /*----------------------- clicked on the edit box --------------------*/
      case EDITBOX:
        x = Event.m_x - edit_x - wnp->pane.x_min - wnp->inside;
        y = Event.m_y - edit_y - wnp->pane.y_min - wnp->inside;
        old_col = old_row = -1;
        if ( range(0, y, (scan_lines - 1)) )
        {
          undo_stack(font + (c * scan_lines), scan_lines);
          tmp = *(font + (c * scan_lines) + cy);
          draw_edit_line(edit_x, edit_y, cy, att, wnp, tmp);
          cx = x / 2, cy = y;
          if (Event.m_button == LB)
            doset = ON;
          else
            doset = OFF;
          do
          {
            m_pos(&m);
            if ( (m.col != old_col) || (m.row != old_row) )
            {
              x = m.col - edit_x - wnp->pane.x_min - wnp->inside;
              y = m.row - edit_y - wnp->pane.y_min - wnp->inside;
              if ( range(0, y, (scan_lines - 1)) && range(0, x, 15) )
              {
                cx = x / 2, cy = y;
                tmp = *(font + (c * scan_lines) + cy);
                if (!doset)
                  tmp &= ~Bits[7 - cx];
                else
                  tmp |=  Bits[7 - cx];
                *(font + (c * scan_lines) + cy) = tmp;
                draw_edit_line(edit_x, edit_y, cy, att, wnp, tmp);
                old_col = m.col, old_row = m.row;
              }
            }
          } while (m.button_status != 0);
          disp_flag = ON;
        }
        break;

      /*------------------------------ copy --------------------------------*/
      case U_COPY:
        movmem(font + (c * scan_lines), save, scan_lines);
        break;

      /*------------------------------ paste -------------------------------*/
      case U_PASTE:
        movmem(save, font + (c * scan_lines), scan_lines);
        disp_flag = ON;
        break;

      /*---------------------------- clear cell ----------------------------*/
      case U_CLR:
        undo_stack(font + (c * scan_lines), scan_lines);
        setmem(font + (c * scan_lines), scan_lines, 0x00 );
        disp_flag = ON;
        break;

      /*----------------------------- fill cell ----------------------------*/
      case U_FILL:
        undo_stack(font + (c * scan_lines), scan_lines);
        setmem(font + (c * scan_lines), scan_lines, 0xff );
        disp_flag = ON;
        break;

      /*----------------------------- not cell -----------------------------*/
      case U_NOT:
        undo_stack(font + (c * scan_lines), scan_lines);
        p = font + (c * scan_lines);
        for( y = 0; y < scan_lines; y++, p++ )
          *p = ~*p;
        disp_flag = ON;
        break;

      /*----------------------------- undo cell ----------------------------*/
      case U_UNDO:
        undo_pop(font + (c * scan_lines), scan_lines);
        disp_flag = ON;
        break;

      /*----------------------------- rotate 90 ----------------------------*/
      case U_ROTATE:
        undo_stack(font + (c * scan_lines), scan_lines);
        movmem(font + (c * scan_lines), save, scan_lines );
        setmem(font + (c * scan_lines), scan_lines, 0x00 );
        p  = save;
        p2 = font + (c * scan_lines);
        for( y = 0; y < 8; y++ )
        {
          tmp = *p++;
          for( x = 0; x < 8; x++ )
          {
            if( tmp & Bits[x] )
              p2[7-x] |= Bits[y];
          }
        }
        disp_flag = ON;
        break;

      /*----------------------- reverse cell X -----------------------------*/
      case REV_X:
        undo_stack(font + (c * scan_lines), scan_lines);
        p = font + (c * scan_lines);
        for( x = 0; x < scan_lines; x++, p++ )
        {
          tmp = *p;
          for( tmp2 = i = 0; i < 8; i++ )
          {
            tmp2 <<= 1;
            tmp2 |= (tmp & 0x01);
            tmp  >>= 1;
          }
          *p = tmp2;
        }
        disp_flag = ON;
        break;

      /*----------------------- reverse cell Y -----------------------------*/
      case REV_Y:
        undo_stack(font + (c * scan_lines), scan_lines);
        movmem(font + (c * scan_lines), save, scan_lines );
        setmem(font + (c * scan_lines), scan_lines, 0x00 );
        p  = save;
        p2 = font + (c * scan_lines);
        for( y = 0; y < scan_lines; y++ )
        {
          tmp = *p++;
          for( x = 0; x < 8; x++ )
          {
            if( tmp & Bits[x] )
              p2[scan_lines-y-1] |= Bits[x];
          }
        }
        disp_flag = ON;

        break;

      /*------------------------ mirror cell X -----------------------------*/
      case MIRR_X:
        undo_stack(font + (c * scan_lines), scan_lines);
        for( y = 0; y < scan_lines; y++ )
        {
          tmp = *(font + (c * scan_lines) + y);
          tmp2 = tmp & 0xf0;
          tmp3 = tmp2 >> 4;
          tmp = 0;
          for( i = 0; i < 4; i++ )
          {
            tmp  <<= 1;
            tmp |= (tmp3 & 0x01);
            tmp3 >>= 1;
          }
          tmp |= tmp2;
          *(font + (c * scan_lines) + y) = tmp;
        }
        disp_flag = ON;
        break;

      /*------------------------ mirror cell Y -----------------------------*/
      case MIRR_Y:
        undo_stack(font + (c * scan_lines), scan_lines);
        for( x = 0; x < 8; x++ )
        {
          p = font + (c * scan_lines);
          for( i = 0; i < scan_lines/2; i++ )
            if( *(p + i) & Bits[7 - x] )
              *(p + (scan_lines - i - 1)) |=  Bits[7 - x];
            else
              *(p + (scan_lines - i - 1)) &= ~Bits[7 - x];
        }
        disp_flag = ON;
        break;

      /*----------------------- insert a column ----------------------------*/
      case C_INS:
        undo_stack(font + (c * scan_lines), scan_lines);
        p = font + (c * scan_lines);
        for( y = 0; y < scan_lines; y++, p++ )
        {
          left  = *p & Maskl[cx];
          right = *p & Maskr[cx];
/*          *p = left | (right >> 1) | (right & 0x80);*/
          *p = left | (right >> 1) | (right & Bits[7 - cx]);
        }
        disp_flag = ON;
        break;

      /*----------------------- delete a column ----------------------------*/
      case C_DEL:
        undo_stack(font + (c * scan_lines), scan_lines);
        p = font + (c * scan_lines);
        for( y = 0; y < scan_lines; y++, p++ )
        {
          left  = *p & Maskl[cx];
          right = *p & Maskr[cx+1];
          *p = left | (right << 1);
        }
        disp_flag = ON;
        break;

      /*------------------------- fill a column ----------------------------*/
      case C_FILL:
        undo_stack(font + (c * scan_lines), scan_lines);
        p = font + (c * scan_lines);
        for( y = 0; y < scan_lines; y++, p++ )
          *p |= Bits[7 - cx];
        disp_flag = ON;
        break;

      /*------------------------ clear a column ----------------------------*/
      case C_CLR:
        undo_stack(font + (c * scan_lines), scan_lines);
        p = font + (c * scan_lines);
        for( y = 0; y < scan_lines; y++, p++ )
          *p &= ~Bits[7 - cx];
        disp_flag = ON;
        break;

      /*------------------------ reverse a column --------------------------*/
      case C_REV:
        undo_stack(font + (c * scan_lines), scan_lines);
        p = font + (c * scan_lines);
        col_mask = 0L;
        for( i = 0; i < scan_lines; i++, p++ )
        {
          col_mask <<= 1;
          if( *p & Bits[7 - cx] )
            col_mask |= 0x01;
        }
        p = font + (c * scan_lines);
        for( i = 0; i < scan_lines; i++, p++ )
        {
          if( col_mask & 0x01 )
            *p |=  Bits[7 - cx];
          else
            *p &= ~Bits[7 - cx];
          col_mask >>= 1;
        }
        disp_flag = ON;
        break;

      /*------------------------- mirror a column --------------------------*/
      case C_MIRR:
        undo_stack(font + (c * scan_lines), scan_lines);
        p = font + (c * scan_lines);
        for( i = 0; i < scan_lines/2; i++ )
          if( *(p + i) & Bits[7 - cx] )
            *(p + (scan_lines - i - 1)) |=  Bits[7 - cx];
          else
            *(p + (scan_lines - i - 1)) &= ~Bits[7 - cx];
        disp_flag = ON;
        break;

      /*--------------------------- not a column ---------------------------*/
      case C_NOT:
        undo_stack(font + (c * scan_lines), scan_lines);
        p = font + (c * scan_lines);
        for( y = 0; y < scan_lines; y++, p++ )
          if( *p & Bits[7 - cx] )
            *p &= ~Bits[7 - cx];
          else
            *p |=  Bits[7 - cx];
        disp_flag = ON;
        break;

      /*------------------------- insert a row -----------------------------*/
      case R_INS:
        undo_stack(font + (c * scan_lines), scan_lines);
        p = font + (c * scan_lines) + cy;
        movmem( p, p + 1, (scan_lines - 1) - cy );
        disp_flag = ON;
        break;

      /*------------------------- insert a row -----------------------------*/
      case R_DEL:
        undo_stack(font + (c * scan_lines), scan_lines);
        p = font + (c * scan_lines) + cy;
        movmem( p + 1, p, (scan_lines - 1) - cy );
        *(font + (c * scan_lines) + (scan_lines - 1)) = 0;
        disp_flag = ON;
        break;

      /*--------------------------- fill a row -----------------------------*/
      case R_FILL:
        undo_stack(font + (c * scan_lines), scan_lines);
        *(font + (c * scan_lines) + cy) = 0xff;
        disp_flag = ON;
        break;

      /*-------------------------- clear a row -----------------------------*/
      case R_CLR:
        undo_stack(font + (c * scan_lines), scan_lines);
        *(font + (c * scan_lines) + cy) = 0x00;
        disp_flag = ON;
        break;

      /*-------------------------- reverse a row ---------------------------*/
      case R_REV:
        undo_stack(font + (c * scan_lines), scan_lines);
        tmp = *(font + (c * scan_lines) + cy);
        for( tmp2 = i = 0; i < 8; i++ )
        {
          tmp2 <<= 1;
          tmp2 |= (tmp & 0x01);
          tmp  >>= 1;
        }
        *(font + (c * scan_lines) + cy) = tmp2;
        disp_flag = ON;
        break;

      /*--------------------------- mirror a row ---------------------------*/
      case R_MIRR:
        undo_stack(font + (c * scan_lines), scan_lines);
        tmp = *(font + (c * scan_lines) + cy);
        tmp2 = tmp & 0xf0;
        tmp3 = tmp2 >> 4;
        tmp = 0;
        for( i = 0; i < 4; i++ )
        {
          tmp  <<= 1;
          tmp |= (tmp3 & 0x01);
          tmp3 >>= 1;
        }
        tmp |= tmp2;
        *(font + (c * scan_lines) + cy) = tmp;
        disp_flag = ON;
        break;

      /*----------------------------- not a row ----------------------------*/
      case R_NOT:
        undo_stack(font + (c * scan_lines), scan_lines);
        *(font + (c * scan_lines) + cy) = ~*(font + (c * scan_lines) + cy);
        disp_flag = ON;
        break;

      /*--------------------------- install font ---------------------------*/
      case FONT_INSTALL:
        m_hide();
        install_font(font, 1, 0, 256, scan_lines, 1);
        set_block_ab(0, 1);
        m_show();
        break;
      case F_8x8:
        Event.key = KEY_ALT_1;
        event_rtn = 1;
        break;
      case F_8x14:
        Event.key = KEY_ALT_2;
        event_rtn = 1;
        break;
      case F_8x16:
        Event.key = KEY_ALT_3;
        event_rtn = 1;
        break;
      case EVENT_RETURNED:
        event_rtn = 1;
        break;

      /*---------------------- an event not processable --------------------*/
    }
    if( event_rtn )
    {
      switch(Event.key)
      {
        /*--------------- character selection commands -----------------*/
        case       '-': if( c >   0 ) c--; break;
        case       '+': if( c < 255 ) c++; break;
        case  KEY_PGUP: if( c >   7 ) c -= 8; break;
        case  KEY_PGDN: if( c < 248 ) c += 8; break;
        case  KEY_HOME: c = 0;   break;
        case   KEY_END: c = 255; break;

        /*------------ character cell movement commands ----------------*/
        case  KEY_CTRL_LEFT: cx = 0; break;
        case KEY_CTRL_RIGHT: cx = 7; break;
        case KEY_CTRL_T:  cy = 0; break;
        case KEY_CTRL_B:  cy = scan_lines - 1; break;
        case   KEY_LEFT: if( cx > 0 ) cx--; break;
        case  KEY_RIGHT: if( cx < 7 ) cx++; break;
        case     KEY_UP: if( cy > 0 ) cy--; break;
        case     KEY_DN: if( cy < scan_lines - 1 ) cy++; break;

        /*--------------- toggle the state of the bit ------------------*/
        case ' ':
        undo_stack(font + (c * scan_lines), scan_lines);
        tmp = *(font + (c * scan_lines) + cy);
        if(tmp & Bits[7 - cx])
          tmp &= ~Bits[7 - cx];
        else
          tmp |=  Bits[7 - cx];
        *(font + (c * scan_lines) + cy) = tmp;
        break;

        /*----------------- get and set the 8x8 font -------------------*/
      case KEY_ALT_1:
        m_hide();
        strcpy(Fullname, "8x8.FNT");
        rom8x8(0, 1);
        rom8x8(1, 1);
        f = get_font_info( 3, &scan_lines, &rows );
        fp = f, np = font;
        for( x = 0; x < scan_lines * 256; x++ )
          *np++ = *fp++;
        install_font( font, 1, 0, 256, scan_lines, 1 );
        set_block_ab(0,1);
        cx = cy = 0;
        draw_item(Tuip, EDITBOX);
        disp_flag = ON;
        m_show();
        break;

        /*----------------- get and set the 8x14 font ------------------*/
      case KEY_ALT_2:
        m_hide();
        strcpy(Fullname, "8x14.FNT");
        rom8x14(0,1);
        rom8x14(1,1);
        f = get_font_info( 2, &scan_lines, &rows );
        fp = f, np = font;
        for( x = 0; x < scan_lines * 256; x++ )
          *np++ = *fp++;
        install_font( font, 1, 0, 256, scan_lines, 1 );
        set_block_ab(0,1);
        cx = cy = 0;
        draw_item(Tuip, EDITBOX);
        disp_flag = ON;
        m_show();
        break;

        /*----------------- get and set the 8x16 font ------------------*/
      case KEY_ALT_3:
        if( Vga )
        {
          m_hide();
          strcpy(Fullname, "8x16.FNT");
          rom8x16(0,1);
          rom8x16(1,1);
          f = get_font_info( 6, &scan_lines, &rows );
          fp = f, np = font;
          for( x = 0; x < scan_lines * 256; x++ )
            *np++ = *fp++;
          install_font( font, 1, 0, 256, scan_lines, 1 );
          set_block_ab(0,1);
          cx = cy = 0;
          draw_item(Tuip, EDITBOX);
          disp_flag = ON;
          m_show();
        }
        break;

      default:
        if( range(0,Event.key,255) )
          c = Event.key;
        break;
      }
      disp_flag = ON;
    }
    if( (range(1, inx, (Tuip->num_base - 1)))
      && (get_item_type(Tuip,inx) == T_BUTTON) )
    {
      deselect(Tuip, inx);
      draw_item(Tuip, inx);
    }
  }
}
/*** end of do_editor ***/

/***************/
/* ~undo_stack */
/*             **************************************************************/
/* This routine puts a character on the undo stack...                       */
/****************************************************************************/
void undo_stack( uchar *p, int scan_lines )
{
  movmem( &Undo[0][0], &Undo[1][0], sizeof(Undo) - 16 );
  movmem( p, &Undo[0][0], scan_lines );                     /* save cell    */
  if( Undo_inx < MAX_UNDO_LEVEL - 1 )
    Undo_inx++;
}
/*** end of undo_stack ***/

/*************/
/* ~undo_pop */
/*           ****************************************************************/
/* This routine pops a character off of the undo stack...                   */
/****************************************************************************/
void undo_pop( uchar *p, int scan_lines )
{
  if( Undo_inx > 0 )
  {
    movmem( &Undo[0][0], p, scan_lines );                   /* restore cell */
    movmem( &Undo[1][0], &Undo[0][0], sizeof(Undo) - 16 );
    Undo_inx--;
  }else{
    tone(1024,10);
  }
}
/*** end of undo_pop ***/

/*************/
/* ~fonthelp */
/*           ****************************************************************/
/* invoke help with the proper search string...                             */
/****************************************************************************/
void fonthelp( int inx )
{
  char *p;

  switch(inx)
  {
    case    FONT_LOAD: p = "/// Loading a Font"; break;
    case FONT_SAVE_AS:
    case    FONT_SAVE: p = "/// Saving a Font";  break;
    case    FONT_EXIT: p = "/// Exiting the Font Editor";       break;
    case      FONTBOX: p = "/// Selecting a Character to Edit"; break;
    case      EDITBOX: p = "/// Using EnQue's Font Editor";     break;
    case       U_FILL: p =  "CF1 : Fill";     break;
    case        U_CLR: p =  "CF2 : Clear";    break;
    case        U_NOT: p =  "CF3 : Not";      break;
    case       U_COPY: p =  "CF4 : Copy";     break;
    case      U_PASTE: p =  "CF5 : Paste";    break;
    case     U_ROTATE: p =  "CF6 : Rotate";   break;
    case        REV_X: p =  "CF7 : ReverseX"; break;
    case        REV_Y: p =  "CF8 : ReverseY"; break;
    case       MIRR_X: p =  "CF9 : MirrorX";  break;
    case       MIRR_Y: p =  "CF10: MirrorY";  break;
    case        C_INS: p =  "F1 : Insert";    break;
    case        C_DEL: p =  "F2 : Delete";    break;
    case       C_FILL: p =  "F3 : Fill";      break;
    case        C_CLR: p =  "F4 : Clear";     break;
    case        C_REV: p =  "F5 : Reverse";   break;
    case       C_MIRR: p =  "F6 : Mirror";    break;
    case        C_NOT: p =  "F7 : Not";       break;
    case        R_INS: p = "AF1 : Insert";    break;
    case        R_DEL: p = "AF2 : Delete";    break;
    case       R_FILL: p = "AF3 : Fill";      break;
    case        R_CLR: p = "AF4 : Clear";     break;
    case        R_REV: p = "AF5 : Reverse";   break;
    case       R_MIRR: p = "AF6 : Mirror";    break;
    case        R_NOT: p = "AF7 : Not";       break;
    case       U_UNDO: p = "/// Undo";        break;
    case FONT_INSTALL: p = "/// Installing a Font";  break;
    case        F_8x8:
    case       F_8x14:
    case       F_8x16: p = "/// Rom Fonts";   break;
              default: p = "Welcome";         break;
  }
  tui_help(1, 3, 78, 18, "fontedit.hlp", p, 0);
  csr_hide();
}
/*** end of fonthelp ***/

/*----------------------------- linkable TUI -------------------------------*/
/**********************/
/* ~register_font_tui */
/*                    *******************************************************/
/* Call this function to register the TUI.  This replaces load_tui!         */
/****************************************************************************/
int register_font_tui( TUI *tuip )
{
  Tui_error = OFF;
  setmem(tuip, sizeof(TUI), 0);
  tuip->version_num = 160L;
  tuip->blink_set = ON;
  tuip->rows = 25;
  tuip->back_att = 0x03;
  tuip->back_bdr_att = 0x03;
  movmem(Palette_font, tuip->palette, 16);

  register_font_dlg0(tuip);
  set_tui_parenting(tuip);
  if (Tui_error)
  {
    free_tui(tuip);
    return(0);
  } else return(1);
}
/*** end of register_font_tui ***/

/***********************/
/* ~register_font_dlg0 */
/*                     ******************************************************/
/* Internally called function to register one dialog.                       */
/****************************************************************************/
void register_font_dlg0( TUI *tuip )
{
  add_dialog(tuip, 0x0000, 0, 1, 80, 23, 0x78, 0x71, 0);
  add_box(tuip, 0, 0x0000, 0, 42, 0, 34, 1, 0x4F, 1);
  add_box(tuip, 0, 0x0000, 0, 43, 12, 14, 5, 0x70, 1);
  add_box(tuip, 0, 0x0000, 0, 23, 12, 14, 7, 0x70, 1);
  add_box(tuip, 0, 0x0000, 0, 43, 3, 14, 8, 0x70, 1);
  add_box(tuip, 0, 0x0000, 0, 23, 3, 14, 8, 0x70, 1);
  add_box(tuip, 0, 0x0009, 0, 60, 3, 16, 16, 0x4F, 1);
  add_box(tuip, 0, 0x0009, 0, 4, 3, 16, 16, 0x40, 1);
  add_button(tuip, 0, 0x0000, 0, 4, 2, 10, 1, 0,
    0x70, 0x70, 0x03, 0x0e, 0x0f, 0, "Character:");
  add_button(tuip, 0, 0x0000, 0, 60, 2, 16, 1, 0,
    0x70, 0x70, 0x03, 0x0e, 0x0f, 0, "Current Font Set");
  add_button(tuip, 0, 0x0000, 0, 43, 12, 14, 1, 0,
    0x17, 0x17, 0x03, 0x0e, 0x0f, 0, "Cell (More)");
  add_button(tuip, 0, 0x0000, 0, 23, 3, 14, 1, 0,
    0x17, 0x17, 0x03, 0x0e, 0x0f, 0, "Column");
  add_button(tuip, 0, 0x0000, 0, 43, 3, 14, 1, 0,
    0x17, 0x17, 0x03, 0x0e, 0x0f, 0, "Row");
  add_button(tuip, 0, 0x0000, 0, 23, 12, 14, 1, 0,
    0x17, 0x17, 0x03, 0x0e, 0x0f, 0, " Cell");
  add_button(tuip, 0, 0x0000, 0, 4, 0, 34, 1, 1,
    0x17, 0x17, 0x17, 0x17, 0x17, 0, "EnQue Font Editor v1.60");
  add_button(tuip, 49, 0x0017, 0, 43, 0, 8, 1, 0,
    0x47, 0x43, 0x40, 0x0e, 0x40, 1, "A1)8x8");
  add_button(tuip, 50, 0x0017, 0, 55, 0, 8, 1, 0,
    0x47, 0x43, 0x40, 0x0e, 0x40, 1, "A2)8x14");
  add_button(tuip, 51, 0x0017, 0, 67, 0, 8, 1, 0,
    0x47, 0x43, 0x40, 0x0e, 0x40, 1, "A3)8x16");
  add_button(tuip, 315, 0x0017, 0, 23, 4, 14, 1, 0,
    0x47, 0x47, 0x20, 0x20, 0x00, 0, "Insert    F1");
  add_button(tuip, 316, 0x0017, 0, 23, 5, 14, 1, 0,
    0x47, 0x47, 0x20, 0x20, 0x00, 0, "Delete    F2");
  add_button(tuip, 317, 0x0017, 0, 23, 6, 14, 1, 0,
    0x47, 0x47, 0x20, 0x20, 0x00, 0, "Fill      F3");
  add_button(tuip, 318, 0x0017, 0, 23, 7, 14, 1, 0,
    0x47, 0x47, 0x20, 0x20, 0x00, 0, "Clear     F4");
  add_button(tuip, 319, 0x0017, 0, 23, 8, 14, 1, 0,
    0x47, 0x47, 0x20, 0x20, 0x00, 0, "Reverse   F5");
  add_button(tuip, 320, 0x0017, 0, 23, 9, 14, 1, 0,
    0x47, 0x47, 0x20, 0x20, 0x00, 0, "Mirror    F6");
  add_button(tuip, 321, 0x0017, 0, 23, 10, 14, 1, 0,
    0x47, 0x47, 0x20, 0x20, 0x00, 0, "Not       F7");
  add_button(tuip, 360, 0x0017, 0, 43, 4, 14, 1, 0,
    0x47, 0x47, 0x20, 0x20, 0x00, 0, "Insert   AF1");
  add_button(tuip, 361, 0x0017, 0, 43, 5, 14, 1, 0,
    0x47, 0x47, 0x20, 0x20, 0x00, 0, "Delete   AF2");
  add_button(tuip, 362, 0x0017, 0, 43, 6, 14, 1, 0,
    0x47, 0x47, 0x20, 0x20, 0x00, 0, "Fill     AF3");
  add_button(tuip, 363, 0x0017, 0, 43, 7, 14, 1, 0,
    0x47, 0x47, 0x20, 0x20, 0x00, 0, "Clear    AF4");
  add_button(tuip, 364, 0x0017, 0, 43, 8, 14, 1, 0,
    0x47, 0x47, 0x20, 0x20, 0x00, 0, "Reverse  AF5");
  add_button(tuip, 365, 0x0017, 0, 43, 9, 14, 1, 0,
    0x47, 0x47, 0x20, 0x20, 0x00, 0, "Mirror   AF6");
  add_button(tuip, 366, 0x0017, 0, 43, 10, 14, 1, 0,
    0x47, 0x47, 0x20, 0x20, 0x00, 0, "Not      AF7");
  add_button(tuip, 350, 0x0017, 0, 23, 13, 14, 1, 0,
    0x47, 0x47, 0x20, 0x20, 0x00, 0, "Fill     CF1");
  add_button(tuip, 351, 0x0017, 0, 23, 14, 14, 1, 0,
    0x47, 0x47, 0x20, 0x20, 0x00, 0, "Clear    CF2");
  add_button(tuip, 352, 0x0017, 0, 23, 15, 14, 1, 0,
    0x47, 0x47, 0x20, 0x20, 0x00, 0, "Not      CF3");
  add_button(tuip, 353, 0x0017, 0, 23, 16, 14, 1, 0,
    0x47, 0x47, 0x20, 0x20, 0x00, 0, "Copy     CF4");
  add_button(tuip, 354, 0x0017, 0, 23, 17, 14, 1, 0,
    0x47, 0x47, 0x20, 0x20, 0x00, 0, "Paste    CF5");
  add_button(tuip, 355, 0x0017, 0, 23, 18, 14, 1, 0,
    0x47, 0x47, 0x20, 0x20, 0x00, 0, "Rotate   CF6");
  add_button(tuip, 356, 0x0017, 0, 43, 13, 14, 1, 0,
    0x47, 0x47, 0x20, 0x20, 0x00, 0, "ReverseX CF7");
  add_button(tuip, 357, 0x0017, 0, 43, 14, 14, 1, 0,
    0x47, 0x47, 0x20, 0x20, 0x00, 0, "ReverseY CF8");
  add_button(tuip, 358, 0x0017, 0, 43, 15, 14, 1, 0,
    0x47, 0x47, 0x20, 0x20, 0x00, 0, "MirrorX  CF9");
  add_button(tuip, 359, 0x0017, 0, 43, 16, 14, 1, 0,
    0x47, 0x47, 0x20, 0x20, 0x00, 0, "MirrorY CF10");
  add_button(tuip, 294, 0x0017, 0, 4, 20, 10, 1, 1,
    0x31, 0x34, 0x37, 0x20, 0x37, 0, "Load");
  add_button(tuip, 287, 0x0017, 0, 16, 20, 10, 1, 1,
    0x31, 0x34, 0x37, 0x20, 0x37, 0, "Save");
  add_button(tuip, 286, 0x0017, 0, 28, 20, 11, 1, 1,
    0x31, 0x34, 0x37, 0x20, 0x37, 5, "Save as");
  add_button(tuip, 278, 0x0017, 0, 44, 18, 11, 1, 1,
    0x20, 0x24, 0x27, 0x20, 0x27, 0, "Undo");
  add_button(tuip, 279, 0x0017, 0, 44, 20, 11, 1, 1,
    0x20, 0x24, 0x27, 0x20, 0x27, 0, "Install");
  add_button(tuip, 274, 0x0017, 0, 60, 20, 17, 1, 1,
    0x17, 0x13, 0x20, 0x20, 0x00, 0, "Exit");
}
/*** end of register_font_dlg0 ***/

/**** END OF FILE ****/
