/****************************************************************************/
/*                                                                          */
/* TD.H                                                                     */
/*                                                                          */
/* The common header files and defines for the EnCom Term Demo program.     */
/*                                                                          */
/****************************************************************************/
#include <stdio.h>

#define   INT_RATE    0x3333                     /* for init_clock          */
#define   TICS_SEC    91                         /* clock tics in 1 second  */

/*---------------------------- terminal defines ----------------------------*/
#define   VT100       0                          /* terminal types          */

#define   NORMAL      ((BLACK << 4) | LIGHTGRAY) /* base attribute states   */
#define   UNDERLINE   0x01
#define   REVERSE     0x70
#define   INVISIBLE   0x00
#define   HIGH_INT    0x08                       /* added attribute states  */
#define   BLINK       128

#define   ANSI_S      0                          /* character set defines   */
#define   ANSI_U      0
#define   ANSI_G      1

/*--------------------------- file transfer defines ------------------------*/
#define   ASCII       1
#define   XMODEM      2
#define   YMODEM      3

#define NUL      0x00
#define SOH      0x01
#define STX      0x02
#define ETX      0x03
#define EOT      0x04
#define ENQ      0x05
#define ACK      0x06
#define BEL      0x07
#define BS       0x08
#define HT       0x09
#define LF       0x0a
#define VT       0x0b
#define FF       0x0c
#define CR       0x0d
#define SO       0x0e
#define SI       0x0f
#define DLE      0x10
#define DC1      0x11
#define DC2      0x12
#define DC3      0x13
#define DC4      0x14
#define NAK      0x15
#define SYN      0x16
#define ETB      0x17
#define CAN      0x18
#define EM       0x19
#define SUB      0x1a
#define ESC      0x1b
#define FS       0x1c
#define GS       0x1d
#define RS       0x1e
#define US       0x1f

#define X_CKSUM     1
#define X_CRC       2
#define ACK_RETRY   16
#define SYNC_RETRY  10

#define X_INIT    0
#define X_OK      1
#define X_NAK     2
#define X_NODATA  3
#define X_PKTNUM  4
#define X_PKTSEQ  5
#define X_CK_ERR  6
#define X_CRC_ERR 7
#define X_CAN     8
#define X_FOPEN   9
#define X_FEMPTY 10
#define X_FREAD  11
#define X_FWRITE 12
#define X_NOMEM  13
#define X_NOACK  14
#define X_NOEOT  15
#define X_NOSOH  16
#define X_BADSOH 17
#define X_DUPPKT 18
#define X_ABORT  19

#define XFER_BUFF_SIZE      16*1024
#define ASCII_PKT_SZ        8

#define XMODEM_PKT_SZ       128
#define YMODEM_PKT_SZ       1024
#define MAX_PKTS            (XFER_BUFF_SIZE/XMODEM_PKT_SZ)

/*---------------------------- transfer structure --------------------------*/
typedef struct xfer_struct
{
  int     rx_mode;              /* receive mode                             */
  int     rx_state;             /* state of transfer program-receive        */
  ulong   rx_chars;             /* characters received                      */
  FILE    *rx_fp;               /* file pointer for transfer-receive        */
  int     rx_func_inx;          /* transfer function index  -receive        */

  int     tx_mode;              /* transmit mode                            */
  int     tx_state;             /* state of transfer program-transmit       */
  ulong   tx_chars;             /* characters transmitted                   */
  FILE    *tx_fp;               /* file pointer for transfer-transmit       */
  int     tx_func_inx;          /* transfer function index  -transmit       */

  int     abort;                /* flag for user request abort              */

  uint    num_pkts;             /* number of packets allocated              */
  ulong   tot_pkts;             /* total number of packets in file          */
  ulong   pkts_mvd;             /* total number of packets txd/rxd          */
  ulong   bytes_mvd;            /* total number of bytes   txd/rxd          */
  uchar   pkt_no;               /* current packet number (% 256)            */
  uchar   resend;               /* packet resend flag/count                 */
  uchar   ck_type;              /* check value type (cksum or CRC)          */
  uchar   sync_cnt;             /* # of tries with this synch char          */
  uchar   sync_char;            /* synch char : 'C' for CRC, NAK for CKSUM  */
  uchar   sync_flag;            /* set to one once sync is established      */

  uchar   rx_tx_flag;           /* 0 = resting, 1 = RX, 2 = TX              */

  uchar   *pkts;                /* area to store data packets               */
  uchar   cksum[MAX_PKTS];      /* current packet checksum (% 256)          */
  uint    crc[MAX_PKTS];        /* current packet CRC (16 bit)              */
  uint    pkt_sizes[MAX_PKTS];  /* current packet size                      */
  uint    pkt_curr;             /* index into above arrays                  */
  uint    pkt_cnt;              /* number of full packets in above array    */
  uint    pkt_size;             /* packet size                              */

  int     status;               /* status of xfer, 0 or 1 for ON or OFF     */
  int     msg;                  /* last message (ACK, CANCEL, DISK_ERR, etc)*/
  int     err_cnt;              /* number of errors that occured            */
  clock_t xtime;                /* used for timeouts                        */
  clock_t stime;                /* time int Tics that transfer started      */
  ulong   fsize;                /* size of file                             */
  char    fname[81];            /* file name                                */

  void    *usr_ptr;             /* user expansion pointer                   */
} XFER;

/*----------------------- structure for the terminal -----------------------*/
typedef struct terminal_struct
{
  PORT            *portp;                 /* pointer to communications port */
  XFER            *xferp;                 /* pointer to file xfer structure */
  int             term_type;              /* the terminal type              */
  int             t_state;                /* current terminal state         */
  int             t_pi;                   /* terminal parameter index       */
  uchar           t_p[16];                /* terminal parameters            */
  uchar           t_ps[16];               /* terminal parameters states     */
  int             csr_x_save;             /* cursor x save area             */
  int             csr_y_save;             /* cursor y save area             */
  int             att_save;               /* attribute save area            */
  int             char_set_save;          /* chracter set save area         */
  int             g0_set;                 /* current g0 character set       */
  int             g1_set;                 /* current g1 character set       */
  int             curr_set;               /* active character set           */
  int             csr_key_mode;           /* cursor key mode on/off         */
  int             norm_att;               /* terminal's NORMAL attribute    */
#ifdef __cplusplus
  int             (*vid_func)(int, WINDOW*);  /* terminal output function   */
  void            (*key_func)(int, WINDOW*);  /* terminal input function    */
#else
  int             (*vid_func)(int, WINDOW*);  /* terminal output function   */
  void            (*key_func)(int, WINDOW*);  /* terminal input function    */
#endif

  unsigned        echo        : 1;        /* 1 if character locally echoed  */
  unsigned        status_on   : 1;        /* 1 if status window open!       */
  unsigned        unused      : 6;
} TERM;


/*------------------------ prototypes for td_main.c ------------------------*/
void  title_screen( WINDOW *wnp );
void  clear_selected( TUI *tuip, int start, int end );
void  com_parms( TUI *tuip, int dlg_inx );
void  dial_number( PORT *portp );
void  show_alert( char *alert );
int   get_response( char *question );
void  status_line( WINDOW *wnp );
void  msg_line( char *msg, WINDOW *wnp );
void  run_dos( char *dos_cmd );
int   register_td_tui( TUI *tuip );

/*------------------------ prototypes for td_term.c ------------------------*/
int   init_term_window( WINDOW *wnp, int port, long baud );
void  kill_term_window( WINDOW *wnp );
void  clear_term_window(void);
void  term_parms( TUI *tuip, int dlg_inx );
int   vt100_ch( int c, WINDOW *wnp );
void  vt100_key( int c, WINDOW *wnp );
void  wn_fgnd( int att, WINDOW *wnp );
void  wn_bkgnd( int att, WINDOW *wnp );
void  set_atts( WINDOW *wnp );
void  clear_t_parms( TERM *termp );
void  term_ctrl( int mode, WINDOW *wnp );
void  save_state( WINDOW *wnp );
void  restore_state( WINDOW *wnp );
void  set_term_type( int type, WINDOW *wnp );
void  term_event( WINDOW *wnp );
int   term_upd(void);


/*------------------------ prototypes for td_xfer.c ------------------------*/
void  finished_sound(void);
void  ascii_upload( TERM *termp, char *fname );
void  ascii_download( TERM *termp, char *fname );
void  ascii_status( TERM *termp, WINDOW *wnp );
int   save_xpkts( FILE *fp, XFER *xferp );
int   make_xpkts( FILE *fp, XFER *xferp );
uchar calc_cksum( uchar *p, int size );
void  xy_modem_download( TERM *termp, int mode, char *fname );
void  xy_modem_upload( TERM *termp, int mode, char *fname );
void  xy_modem_status( TERM *termp, WINDOW *wnp, int mode );
void  upload_file( TUI *tuip, int dlg_inx );
void  download_file( TUI *tuip, int dlg_inx );


/*** END OF FILE ***/
